package br.ufes.inf.lp.ppgicred.dominio

import java.util.Date
import java.util.Calendar

class Docente (val codigo: BigInt, val nome: String, val nascimento: Date, val ingresso: Date, val coord: Boolean) {
  private[this] var listaPublicacoes: List[Publicacao] = List[Publicacao]()

  // Adiciona uma nova publicação no início da lista de publicações.
  def adicionaPublicacao (p: Publicacao): Unit = listaPublicacoes = p :: listaPublicacoes

  def getIdate (atual: Date): Int = {
    val dob = Calendar.getInstance
    val today = Calendar.getInstance()

    dob.setTime(nascimento)
    today.setTime(atual)

    var age = today.get(Calendar.YEAR) - dob.get(Calendar.YEAR)

    // Se o dia do ano corrente for menor que o dia dá DoB (date of birth), ele ainda não fez
    // aniversário no ano corrente.
    if (today.get(Calendar.DAY_OF_YEAR) < dob.get(Calendar.DAY_OF_YEAR))
      age -= 1

    // Retorna a idade.
    age
  }

  def calculaPontuacao (ano: Date, r: Regra): Double = {
    val anoDesejado: Calendar = Calendar.getInstance()
    anoDesejado.setTime(ano)

    val inicioVigencia: Calendar = Calendar.getInstance()
    inicioVigencia.setTime(ano)
    inicioVigencia.add(Calendar.YEAR, -r.anosParaConsiderar)

    // Se a lista de publicações for muito grande, vale mais a pena
    // transformar esse fluxo em um loop foreach e aplicar todas as operações
    // dentro de cada iteração do loop.
    // Porém, a utilização de streams é mais comum em Scala e a legibilidade do
    // código melhora, pos a lógica está bem segmentada.

    // Transforma a lista de publicações em uma stream
    val pontuacao: Double = listaPublicacoes.toStream
      // Filtra para remover as publicações que não estão dentro do período de vigência
      .filter(
        p => {
          val dataPublicacao: Calendar = Calendar.getInstance()
          dataPublicacao.set(Calendar.YEAR, p.ano)
          dataPublicacao.set(Calendar.MONTH, 1)
          dataPublicacao.set(Calendar.DAY_OF_MONTH, 1)

          !dataPublicacao.before(inicioVigencia) && dataPublicacao.getTime.before(ano)
        }
      )
      // Cria uma lista com a nota de cada publicação
      .map(
        p => {
          val multiplicador: Double = p match {
            case pp: PublicacaoPeriodico => r.multiplicadorPeriodico
            case _ => 1.0d
          }

          multiplicador * r.getPontuacaoQualis(p.getQualificacaoVeiculoNoAno(p.ano))
        }
      )
      // Soma a lista de notas e retorna o resultado
      .sum

    pontuacao
  }
}
