package br.ufes.inf.lp.ppgicred.dominio

// Atributos declarados com `val` no construtor possuem getter.
class Publicacao (val ano: Int, val numero: Int, val paginaInicial: Int, val paginaFinal: Int, val titulo: String, listaDocentes: List[Docente]) extends Comparable[Publicacao] with Serializable {

  //Este tipo de declaração cria um objeto não inicializado (seu valor é null).
  private[this] var veiculo: Veiculo = _

  def setVeiculo (v: Veiculo): Unit = {
    if(veiculo == null) veiculo = v
  }

  def getNumeroDocentes: Int = listaDocentes.size

  def getSiglaVeiculo: String = veiculo.sigla

  def getQualificacaoVeiculoNoAno (ano: Int): String = veiculo.getQualificacaoAno(ano)

  def getNomeVeiculo: String = veiculo.nome

  def getFatorImpactoVeiculo: Double = veiculo.fatorImpacto

  // Transforma a lista em uma string com os nomes separados por vírgula (join(","))
  def getNomeDocentes: String = listaDocentes.toStream.map(_.nome).mkString(",")

  override def compareTo (o: Publicacao): Int = {
    val c1 = o.ano compareTo ano
    
    if(c1 == 0) {
      val c2 = veiculo.sigla compareTo o.getSiglaVeiculo

      if(c2 == 0) {

        titulo compareTo o.titulo
      } else c2
    } else c1
  }
}
