package br.ufes.inf.lp.ppgicred.dominio

import scala.collection.mutable

// Atributos declarados com `val` no construtor possuem getter.
class Veiculo (val sigla: String, val nome: String, val tipo: Char, val fatorImpacto: Double, val issn: String) extends Serializable {

  // Atributos privados para a classe.
  private[this] var mapaQualificacoes: mutable.TreeMap[Int, Qualificacao] = mutable.TreeMap[Int, Qualificacao]()
  private[this] var listaPublicacoes: List[Publicacao] = List[Publicacao]()

  // Adiciona uma qualificação ao mapa de qualificações.
  def adicionaQualificacao (q: Qualificacao): Unit = mapaQualificacoes += (q.ano -> q)

  // Adiciona uma nova publicação no início da lista de publicações.
  // Esta operação retorna uma nova lista com o elemento adicionado.
  def adicionaPublicacao (p: Publicacao): Unit = listaPublicacoes = p :: listaPublicacoes

  def getQualificacaoAno (ano: Int): String = {
    var r: String = null

    // Transforma o mapa em uma stream
    r = mapaQualificacoes.toStream
      // Filtra todas as qualificações anteriores ao ano desejado
      .filter(_._1 <= ano)
      // Cria uma lista contendo apenas os nomes das qualificações
      .map(_._2.qualis)
      // É possível que não exista uma qualificação anterior ao ano desejado.
      // lastOption retorna um Option[String], similar ao Optional de Java
      .lastOption
      // orNull é equivalente ao Option[String].getOrElse(null)
      .orNull

    if(r == null)
      throw new RuntimeException(s"Qualificação anterior ao ano $ano para o veículo $nome não existe!")

    // Retorna a qualificação encontrada
    r
  }
}
