package br.ufes.inf.lp.ppgicred.es

import java.util.Date

import br.ufes.inf.lp.ppgicred.dominio.{Docente, Publicacao, Regra, Veiculo}

import scala.collection.mutable

class DadosEntrada {
  private[this] var mapaDocentesNome: Map[String, Docente] = Map[String, Docente]()
  private[this] var mapaDocentesCodigo: Map[BigInt, Docente] = Map[BigInt, Docente]()
  private[this] var mapaVeiculos: mutable.TreeMap[String, Veiculo] = mutable.TreeMap[String, Veiculo]()
  private[this] var listaPublicacoes: List[Publicacao] = List[Publicacao]()
  private[this] var listaRegras: List[Regra] = List[Regra]()
  private[this] var ano: Int = _

  def parseAno (anoString: String): Unit = ano = anoString.toInt

  def adicionaDocente (d: Docente): Unit = {
    mapaDocentesNome += (d.nome -> d)
    mapaDocentesCodigo += (d.codigo -> d)
  }

  def adicionaPublicacao (p: Publicacao): Unit = listaPublicacoes = p :: listaPublicacoes

  def adicionaRegra (r: Regra): Unit = listaRegras = r :: listaRegras

  def adicionaVeiculo (v: Veiculo): Unit = mapaVeiculos += (v.sigla -> v)

  def codigoDocenteExiste (codigo: Int): Boolean = mapaDocentesCodigo.contains(codigo)

  def getDocenteCodigo (codigo: BigInt): Docente = mapaDocentesCodigo(codigo)

  def getDocenteNome (nome: String): Docente = mapaDocentesNome(nome)

  def siglaVeiculoExiste (sigla: String): Boolean = mapaVeiculos.contains(sigla)

  def getVeiculo (sigla: String): Veiculo = mapaVeiculos(sigla)

  def getAno: Int = ano

  def getListaPublicacoesQualis (qualis: String): List[Publicacao] = listaPublicacoes.toStream.filter(p => { p.getQualificacaoVeiculoNoAno(p.ano) == qualis }).toList

  def getNomesMapaDocentes: List[String] = mapaDocentesNome.keys.toList

  def getRegraVigencia (d: Date): Regra = listaRegras.toStream.find(r => { d.getTime >= r.inicio.getTime && d.getTime <= r.fim.getTime }).orNull
}
