package br.ufes.inf.lp.ppgicred.es

import java.io._
import java.util.Calendar._
import java.util.{Calendar, Locale}

import br.ufes.inf.lp.ppgicred.dominio.{Docente, Publicacao, Regra}

class Saida (qualisOrdenados: List[String], recredenciamento: String, publicacoes: String) {
  private val locale: Locale = new Locale("pt", "BR")

  def geraRelatorioPublicacoes (ppgi: DadosEntrada): Unit =  {
    val target = new BufferedWriter(new OutputStreamWriter(new FileOutputStream(publicacoes), "utf-8"))

    try {
      target.write("Ano;Sigla Veículo;Veículo;Qualis;Fator de Impacto;Título;Docentes\n")


      val publicacaoOrdering = implicitly[Ordering[Publicacao]]
      import publicacaoOrdering._

      qualisOrdenados.foreach(qualis => {
        val listaPublicacoes = ppgi.getListaPublicacoesQualis(qualis).sorted

        listaPublicacoes.map(publicacao => {
          s"${publicacao.ano};${publicacao.getSiglaVeiculo};${publicacao.getNomeVeiculo};${publicacao.getQualificacaoVeiculoNoAno(publicacao.ano)};" +
            s"${"%.3f".formatLocal(locale, publicacao.getFatorImpactoVeiculo)};${publicacao.titulo};${publicacao.getNomeDocentes}\n"
        }).foreach(target.write)
      })
    } finally
      target.close()
  }

  def geraRelatorioRecredenciamento (ppgi: DadosEntrada): Unit = {
    val anoCalculo = Calendar.getInstance()
    anoCalculo.set(YEAR, ppgi.getAno)
    anoCalculo.set(MONTH, 1)
    anoCalculo.set(DAY_OF_MONTH, 1)

    val r: Regra = ppgi.getRegraVigencia(anoCalculo.getTime)

    val target = new BufferedWriter(new OutputStreamWriter(new FileOutputStream(recredenciamento), "utf-8"))

    try {
      target.write("Docente;Pontuação;Recredenciado?\n")

      // Processa os docentes, um a um, retornando a string que deve ser exibida no arquivo.
      ppgi.getNomesMapaDocentes
        // Ordena usando o operador menor que (ordem alfabética)
        .sortWith(_ < _)
        // Transforma os nomes dos docentes no resultado esperado
        .map(nomeDocente => {
        val d: Docente = ppgi.getDocenteNome(nomeDocente)
        val pontuacaoDocente = d.calculaPontuacao(anoCalculo.getTime, r)
        val pontuacaoDocenteStr = "%.1f".formatLocal(locale, pontuacaoDocente)

        val dataNovato = Calendar.getInstance()
        dataNovato.setTime(d.ingresso)
        dataNovato.add(YEAR, 3)

        val recredenciado = {
          if (d.coord) "Coordenador"
          else if (!anoCalculo.after(dataNovato)) "PPJ"
          else if (d.getIdate(anoCalculo.getTime) >= 60) "PPS"
          else if (pontuacaoDocente >= r.pontuacaoMinima) "Sim"
          else "Não"
        }

        // Interpola os valores obtidos nos cálculos
        s"${d.nome};$pontuacaoDocenteStr;$recredenciado\n"
      })
        // Chama a função write para cada string retornada no processamento da Stream
        .foreach(target.write)
    } finally
      target.close()
  }

  def salvarRelatorios(ppgi: DadosEntrada): Unit = {
    geraRelatorioPublicacoes(ppgi)
    geraRelatorioRecredenciamento(ppgi)
  }
}

